@@ -12,12 +12,11 @@ use base qw(Net::DNS::Resolver::Base DynaLoader), OS_CONF;
 our $VERSION;
 
 BEGIN {
-	$VERSION = '1.24_01';
+	$VERSION = '1.24_02';
 	eval { __PACKAGE__->bootstrap($VERSION) };
 }
 
 use constant UB_CONTEXT => 'Net::DNS::Resolver::Unbound::Context';
-use constant UB_SEND => UB_CONTEXT->can('ub_send');
 
 
 =head1 NAME
@@ -145,6 +144,8 @@ See L<Net::DNS::Resolver>.
 
 =cut
 
+use constant UB_SEND => UB_CONTEXT->can('ub_send');
+
 sub send {
 	my ( $self, @argument ) = @_;
 	$self->_finalise_config;
@@ -207,16 +208,6 @@ Set Unbound resolver (name,value) context option.
 
 sub option {
 	my ( $self, $name, @value ) = @_;
-	unless (@value) {
-		$self->_finalise_config;
-		my @return = eval {
-			my $optref = $self->{config}->{set_option};
-			my %option = @$optref;
-			my @list = @{$option{"${name}:"}};
-		};
-		return wantarray ? @return : shift @return;
-	}
-
 	return $self->_option( $name, @value );
 }
 
@@ -263,11 +254,11 @@ Use DNS over TLS for queries to nameservers specified using set_fwd().
 
 =cut
 
+use constant SET_TLS => UB_CONTEXT->can('set_tls');
 
 sub set_tls {
 	my ( $self, $tls ) = @_;
-	use constant SET_TLS => UB_CONTEXT->can('set_tls');
-	return $self->_config( 'set_tls', $tls ) if SET_TLS;
+	return SET_TLS ? $self->_config( 'set_tls', $tls ) : undef;
 }
 
 
@@ -281,10 +272,11 @@ and the 'DHCP DNS' IP address, use set_fwd().
 
 =cut
 
+use constant SET_STUB => UB_CONTEXT->can('set_stub');
+
 sub set_stub {
 	my ( $self, $zone, $address, $prime ) = @_;
-	use constant SET_STUB => UB_CONTEXT->can('set_stub');
-	return $self->_config( 'set_stub', [[$zone, $address, $prime]] ) if SET_STUB;
+	return SET_STUB ? $self->_config( 'set_stub', [[$zone, $address, $prime]] ) : undef;
 }
 
 
@@ -362,10 +354,11 @@ trust anchor is changed.
 
 =cut
 
+use constant ADD_TA_AUTR => UB_CONTEXT->can('add_ta_autr');
+
 sub add_ta_autr {
 	my ( $self, $filename ) = @_;
-	use constant ADD_TA_AUTR => UB_CONTEXT->can('add_ta_autr');
-	return $self->_config( 'add_ta_autr', $filename ) if ADD_TA_AUTR;
+	return ADD_TA_AUTR ? $self->_config( 'add_ta_autr', $filename ) : undef;
 }
 
 
@@ -452,11 +445,11 @@ sub string {
 END
 	$self->_finalise_config;
 	my %config = %{$self->{config}};
-	my $optref = $config{set_option};			# expand option list
-	my %option = @{$optref || []};
+	my $optref = $config{set_option} || [];		# sort option list
+	my %option = map { @$_ } @$optref;
 	my @option;
 	foreach my $opt ( sort keys %option ) {
-		push @option, [$opt, $_] for @{$option{$opt}};
+		push @option, [$opt, $option{$opt}];
 	}
 
 	my $format = ";; %s\t%s\n";
@@ -514,20 +507,24 @@ sub _config {
 
 
 sub _option {
-	my ( $self, $name, $arg ) = @_;
+	my ( $self, $name, @arg ) = @_;
 	$self->{ub_ctx} = Net::DNS::Resolver::Unbound::Context->new() unless $self->{update};
 	my $setopt = $self->{config}->{set_option};
-	my $optref = $self->{update}->{set_option} || $setopt;
-	my %option = @{$optref || []};
+	my $updopt = $self->{update}->{set_option} || [];
+	my %option = map { @$_ } @$setopt, @$updopt;
 
 	my $opt = "${name}:";
-	if ($arg) {
-		my $list = $option{$opt} ||= [];
+	return $option{$opt} unless @arg;
+
+	my $arg = shift @arg;
+	delete $option{$opt};
+	if ( defined $arg ) {
 		$self->{ub_ctx}->set_option( $opt, $arg );
-		push @$list, $arg;
-		$self->{update}->{set_option} = [%option];
+		$option{$opt} = $arg;
 	}
 
+	my @option = map { [$_, $option{$_}] } keys %option;
+	$self->{update}->{set_option} = \@option;
 	return;
 }
 
@@ -541,29 +538,20 @@ my %IP_conf = (
 sub _finalise_config {
 	my $self = shift;
 
-	return unless $self->{update};
-
-	foreach my $key ( keys %IP_conf ) {
-		my ( $name, $arg ) = @{$IP_conf{$key}};
-		my @arg = $self->$key ? $arg : ();
-		eval { $self->_option( $name, @arg ); 1 }	# if implemented
-	}
-
-	my $config = $self->{config};
 	my $update = delete $self->{update};
-	my %config = ( %$config, %$update );
-
-	my $optref = $config{set_option};			# expand option list
-	my %option = @{$optref || []};
-	my @option;
-	foreach my $opt ( keys %option ) {
-		push @option, [$opt, $_] for @{$option{$opt}};
-	}
-
+	return unless $update;
 	my $ctx = $self->{ub_ctx} = Net::DNS::Resolver::Unbound::Context->new();
 
+	my $config = $self->{config} || { set_option => [] };	# merge config updates
+	my %config = ( %$config, %$update );
+
+	my $optref = delete $config{set_option};		# create option hash table
+	my %option = map { @$_ } @$optref;
+	my @prefer = keys %IP_conf;
+	delete @option{@prefer};
+	my @option = map { [$_, $option{$_}] } keys %option;	# reassemble set_option list
+
 	foreach my $name ( keys %config ) {			# rebuild unbound context
-		local $config{set_option} = \@option;
 		my $value = $config{$name};
 		if ( ref $value ) {
 			foreach my $arg (@$value) {
@@ -575,17 +563,28 @@ sub _finalise_config {
 		}
 	}
 
+	foreach my $key (@prefer) {				# append IP preference
+		eval {
+			my ( $name, $arg ) = @{$IP_conf{$key}};
+			my $opt = "${name}:";
+			$ctx->set_option($opt,$arg);	# not implemented?
+			push @option, [$opt, $arg];
+		} if $self->$key;
+	}
+
+	$config{set_option} = \@option if @option;
 	$self->{config} = \%config;
 	return;
 }
 
+
 1;
 __END__
 
 
 =head1 COPYRIGHT
 
-Copyright (c)2022,2023 Dick Franks
+Copyright (c)2022,2024 Dick Franks
 
 All Rights Reserved
 
